#include <Gestalt.h>
// #include "GestaltValue.h"
#include "LogInterface.h"
#include <Memory.h>
#include <string.h>
#include <TextUtils.h>
#include <Types.h>

void InsertIntoRing ( LogInterfacePtr logP, const void* data, unsigned long size )
{
	if ( logP && logP->callbackUPP )
		CallLoggerCallbackProc ( logP->callbackUPP, logP,  insertMsg, (long) data, (long) size, (long) 0 );	
}

void InsertHexDumpIntoRing ( LogInterfacePtr logP, const void* data, unsigned long size )
{
	if ( logP && logP->callbackUPP )
		CallLoggerCallbackProc ( logP->callbackUPP, logP,  insertHexDumpMsg, (long) data, (long) size, (long) 0 );	
}

void PrintfRing ( LogInterfacePtr logP, const char* format, ... )
{
	if ( logP && logP->callbackUPP )
	{	va_list args;
		va_start ( args, format);
		CallLoggerCallbackProc ( logP->callbackUPP, logP,  printfRingMsg, (long) format, (long) &args, (long) 0 );
		va_end ( args );
	}
}

void vPrintfRing ( LogInterfacePtr logP, const char* formatStr, va_list* args )
{
	if ( logP && logP->callbackUPP )
		CallLoggerCallbackProc ( logP->callbackUPP, logP, printfRingMsg, (long) formatStr, (long) args, (long) 0 );
}

unsigned long RemoveFromRing ( LogInterfacePtr logP, void* data, unsigned long dataSize )
{	unsigned long result = 0;

	if ( logP && logP->callbackUPP )
		result = CallLoggerCallbackProc ( logP->callbackUPP, logP, removeMsg, (long) data, (long) dataSize, (long) 0 );	
	
	return result;
}

unsigned long GetDataInRingSize ( LogInterfacePtr logP )
{	unsigned long result = 0;

	if ( logP && logP->callbackUPP )
		result = CallLoggerCallbackProc ( logP->callbackUPP, logP, getDataInRingSizeMsg, 0, 0, 0 );	

	return result;
}

Boolean GetRingDataLost ( LogInterfacePtr logP )
{	Boolean result = false;

	if ( logP && logP->callbackUPP )
		result = CallLoggerCallbackProc ( logP->callbackUPP, logP, getRingDataLostMsg, 0, 0, 0 );	

	return result;
}

unsigned long GetRingChangedCount ( LogInterfacePtr logP )
{
	if ( logP )
		return 0;
	else
		return 0;
}

LogInterfacePtr GetLogInterfacePtr ( )
{	LogInterfacePtr result;

	if ( Gestalt ( kLogInterfaceSignature, (long*) & result ) != noErr )
		result = nil;
	
	return result;
}

void ClearText ( LogInterfacePtr logP )
{	
	if ( logP && logP->callbackUPP )
		CallLoggerCallbackProc ( logP->callbackUPP, logP, clearMsg, 0, 0, 0 );	
}

Boolean GetFlagValue ( LogInterfacePtr logP, unsigned long whichFlag )
{	Boolean result = false;

	if ( logP && logP->callbackUPP )
		result = CallLoggerCallbackProc ( logP->callbackUPP, logP, getFlagMsg, whichFlag, 0, 0 );

	return result;
}

void SetFlagValue ( LogInterfacePtr logP, unsigned long whichFlag, Boolean value )
{
	if ( logP && logP->callbackUPP )
		CallLoggerCallbackProc ( logP->callbackUPP, logP, setFlagMsg, whichFlag, value, 0 );
}

void Debug ( char *format, ... )
{	LogInterfacePtr logP = GetLogInterfacePtr();
	va_list		argptr;

	if ( logP )
	{
		va_start ( argptr, format );
		
		vPrintfRing ( logP, format, & argptr );
		
		va_end ( argptr );
	}
}

void DebugF ( unsigned short whichFlag, char *format, ... )
{	LogInterfacePtr logP = GetLogInterfacePtr();
	va_list		argptr;

	if ( logP && GetFlagValue ( logP, whichFlag ) )
	{
		va_start ( argptr, format );
		
		vPrintfRing ( logP, format, & argptr );
		
		va_end ( argptr );
	}
}



#ifndef DONT_DEFINE_CONSOLE_FUNCTIONS
#ifdef __MWERKS__

#ifdef __cplusplus
extern "C" {
#endif

#include "LogInterface.h"

#include <SIOUX.h>

long ReadCharsFromConsole ( char* data, long size );
short SIOUXHandleOneEvent ( struct EventRecord* event );
long WriteCharsToConsole ( char* data, long size );
void RemoveConsole ( );
short InstallConsole ( short fd );
char *__ttyname(long fildes);

long ReadCharsFromConsole ( char* data, long size )
{
	return -1;
}

short SIOUXHandleOneEvent ( struct EventRecord* event )
{
	return false;
}

tSIOUXSettings SIOUXSettings;

long WriteCharsToConsole ( char* data, long size )
{
	if ( GetLogInterfacePtr() )
		InsertIntoRing ( GetLogInterfacePtr(), data, size );
	return size;
}

void RemoveConsole ( )
{
}

short InstallConsole ( short fd )
{
	return 0;
}

char *__ttyname(long fildes)
{
#pragma unused (fildes)
	/* all streams have the same name */
	static char *__devicename = "null device";

	if (fildes >= 0 && fildes <= 2)
		return (__devicename);

	return (0L);
}


#ifdef __cplusplus
};
#endif

#endif

#endif